@@ -41,11 +41,11 @@
 #' @export
 add_row <- function(.data, ..., .before = NULL, .after = NULL) {
   if (inherits(.data, "grouped_df")) {
-    cnd_signal(error_add_rows_to_grouped_df())
+    abort_add_rows_to_grouped_df()
   }
 
   if (!is.data.frame(.data)) {
-    deprecate_warn("2.1.1", "add_row(.data = 'must be a data frame')")
+    deprecate_stop("2.1.1", "add_row(.data = 'must be a data frame')")
   }
 
   if (dots_n(...) == 0L) {
@@ -57,7 +57,7 @@ add_row <- function(.data, ..., .before = NULL, .after = NULL) {
 
   extra_vars <- setdiff(names(df), names(.data))
   if (has_length(extra_vars)) {
-    cnd_signal(error_incompatible_new_rows(extra_vars))
+    abort_incompatible_new_rows(extra_vars)
   }
 
   pos <- pos_from_before_after(.before, .after, nrow(.data))
@@ -120,22 +120,20 @@ rbind_at <- function(old, new, pos) {
 #' # You can't overwrite existing columns
 #' try(df %>% add_column(x = 4:6))
 #'
-
 #' # You can't create new observations
 #' try(df %>% add_column(z = 1:5))
 #'
 #' @export
 add_column <- function(.data, ..., .before = NULL, .after = NULL,
-                       .name_repair = c("check_unique", "unique", "universal", "minimal")) {
-
+                       .name_repair = c("check_unique", "unique", "universal", "minimal", "unique_quiet", "universal_quiet")) {
   if (!is.data.frame(.data)) {
-    deprecate_warn("2.1.1", "add_column(.data = 'must be a data frame')")
+    deprecate_stop("2.1.1", "add_column(.data = 'must be a data frame')")
   }
 
   if (has_length(.data) && (!is_named(.data) || anyDuplicated(names2(.data))) && missing(.name_repair)) {
-    deprecate_warn("3.0.0", "add_column(.data = 'must have unique names')",
-      details = 'Use `.name_repair = "minimal"`.')
-    .name_repair <- "minimal"
+    deprecate_stop("3.0.0", "add_column(.data = 'must have unique names')",
+      details = 'Use `.name_repair = "minimal"`.'
+    )
   }
 
   df <- tibble(..., .name_repair = .name_repair)
@@ -148,7 +146,7 @@ add_column <- function(.data, ..., .before = NULL, .after = NULL,
     if (nrow(df) == 1) {
       df <- df[rep(1L, nrow(.data)), ]
     } else {
-      cnd_signal(error_incompatible_new_cols(nrow(.data), df))
+      abort_incompatible_new_cols(nrow(.data), df)
     }
   }
 
@@ -172,14 +170,14 @@ add_column <- function(.data, ..., .before = NULL, .after = NULL,
 
 # helpers -----------------------------------------------------------------
 
-pos_from_before_after_names <- function(before, after, names) {
+pos_from_before_after_names <- function(before, after, names, call = caller_env()) {
   before <- check_names_before_after(before, names)
   after <- check_names_before_after(after, names)
 
-  pos_from_before_after(before, after, length(names))
+  pos_from_before_after(before, after, length(names), call)
 }
 
-pos_from_before_after <- function(before, after, len) {
+pos_from_before_after <- function(before, after, len, call = caller_env()) {
   if (is.null(before)) {
     if (is.null(after)) {
       len
@@ -190,7 +188,7 @@ pos_from_before_after <- function(before, after, len) {
     if (is.null(after)) {
       limit_pos_range(before - 1L, len)
     } else {
-      cnd_signal(error_both_before_after())
+      abort_both_before_after(call)
     }
   }
 }
@@ -212,7 +210,7 @@ check_names_before_after <- function(j, x) {
 
 check_needs_no_dim <- function(j) {
   if (needs_dim(j)) {
-    cnd_signal(error_dim_column_index(j))
+    abort_dim_column_index(j)
   }
 }
 
@@ -220,37 +218,43 @@ check_names_before_after_character <- function(j, names) {
   pos <- safe_match(j, names)
   if (anyNA(pos)) {
     unknown_names <- j[is.na(pos)]
-    cnd_signal(error_unknown_column_names(unknown_names))
+    abort_unknown_column_names(unknown_names)
   }
   pos
 }
 
 # Errors ------------------------------------------------------------------
 
-error_add_rows_to_grouped_df <- function() {
-  tibble_error("Can't add rows to grouped data frames.")
+msg_unknown_column_names <- function(names) {
+  pluralise_commas("Can't find column(s) ", tick(names), " in `.data`.")
 }
 
-error_incompatible_new_rows <- function(names) {
-  tibble_error(
+abort_add_rows_to_grouped_df <- function(call = caller_env()) {
+  tibble_abort(call = call, "Can't add rows to grouped data frames.")
+}
+
+abort_incompatible_new_rows <- function(names, call = caller_env()) {
+  tibble_abort(
+    call = call,
     problems(
       "New rows can't add columns:",
-      cnd_message(error_unknown_column_names(names))
+      msg_unknown_column_names(names)
     ),
     names = names
   )
 }
 
-error_both_before_after <- function() {
-  tibble_error("Can't specify both `.before` and `.after`.")
+abort_both_before_after <- function(call = caller_env()) {
+  tibble_abort(call = call, "Can't specify both `.before` and `.after`.")
 }
 
-error_unknown_column_names <- function(j, parent = NULL) {
-  tibble_error(pluralise_commas("Can't find column(s) ", tick(j), " in `.data`."), j = j, parent = parent)
+abort_unknown_column_names <- function(j, parent = NULL, call = caller_env()) {
+  tibble_abort(call = call, pluralise_commas("Can't find column(s) ", tick(j), " in `.data`."), j = j, parent = parent)
 }
 
-error_incompatible_new_cols <- function(n, df) {
-  tibble_error(
+abort_incompatible_new_cols <- function(n, df, call = caller_env()) {
+  tibble_abort(
+    call = call,
     bullets(
       "New columns must be compatible with `.data`:",
       x = paste0(
@@ -263,3 +267,8 @@ error_incompatible_new_cols <- function(n, df) {
     actual = nrow(df)
   )
 }
+
+abort_dim_column_index <- function(j, call = caller_env()) {
+  # friendly_type_of() doesn't distinguish between matrices and arrays
+  tibble_abort(call = call, paste0("Must use a vector in `[`, not an object of class ", class(j)[[1]], "."))
+}
