@@ -42,6 +42,10 @@
 #'   * `NA`: keep row names.
 #'   * A string: the name of a new column. Existing rownames are transferred
 #'     into this column and the `row.names` attribute is deleted.
+#'     No name repair is applied to the new column name, even if `x` already contains
+#'     a column of that name.
+#'     Use `as_tibble(rownames_to_column(...))` to safeguard against this case.
+#'
 #'  Read more in [rownames].
 
 #' @param _n,validate
@@ -55,7 +59,7 @@
 #' df <- as_tibble(m)
 as_tibble <- function(x, ...,
                       .rows = NULL,
-                      .name_repair = c("check_unique", "unique", "universal", "minimal"),
+                      .name_repair = c("check_unique", "unique", "universal", "minimal", "unique_quiet", "universal_quiet"),
                       rownames = pkgconfig::get_config("tibble::rownames", NULL)) {
   UseMethod("as_tibble")
 }
@@ -64,9 +68,15 @@ as_tibble <- function(x, ...,
 #' @rdname as_tibble
 as_tibble.data.frame <- function(x, validate = NULL, ...,
                                  .rows = NULL,
-                                 .name_repair = c("check_unique", "unique", "universal", "minimal"),
+                                 .name_repair = c("check_unique", "unique", "universal", "minimal", "unique_quiet", "universal_quiet"),
                                  rownames = pkgconfig::get_config("tibble::rownames", NULL)) {
-  .name_repair <- compat_name_repair(.name_repair, validate, missing(.name_repair))
+  if (!is.null(validate)) {
+    deprecate_stop("2.0.0", "tibble::as_tibble(validate = )", "as_tibble(.name_repair =)")
+  }
+
+  if (!identical(class(x), "data.frame") && !inherits(x, "tbl_df")) {
+    x <- as.data.frame(x)
+  }
 
   old_rownames <- raw_rownames(x)
   if (is.null(.rows)) {
@@ -81,7 +91,8 @@ as_tibble.data.frame <- function(x, validate = NULL, ...,
     attr(result, "row.names") <- old_rownames
     result
   } else {
-    if (length(old_rownames) > 0 && is.na(old_rownames[1L])) { # if implicit rownames
+    if (length(old_rownames) > 0 && is.na(old_rownames[1L])) {
+      # if implicit rownames
       old_rownames <- seq_len(abs(old_rownames[2L]))
     }
     old_rownames <- as.character(old_rownames)
@@ -92,45 +103,29 @@ as_tibble.data.frame <- function(x, validate = NULL, ...,
 #' @export
 #' @rdname as_tibble
 as_tibble.list <- function(x, validate = NULL, ..., .rows = NULL,
-                           .name_repair = c("check_unique", "unique", "universal", "minimal")) {
-  .name_repair <- compat_name_repair(.name_repair, validate, missing(.name_repair))
+                           .name_repair = c("check_unique", "unique", "universal", "minimal", "unique_quiet", "universal_quiet")) {
+  if (!is.null(validate)) {
+    deprecate_stop("2.0.0", "tibble::as_tibble(validate = )", "as_tibble(.name_repair =)")
+  }
 
   lst_to_tibble(x, .rows, .name_repair, col_lengths(x))
 }
 
-lst_to_tibble <- function(x, .rows, .name_repair, lengths = NULL) {
+lst_to_tibble <- function(x, .rows, .name_repair, lengths = NULL, call = caller_env()) {
   x <- unclass(x)
-  x <- set_repaired_names(x, repair_hint = TRUE, .name_repair)
-  x <- check_valid_cols(x)
-  recycle_columns(x, .rows, lengths)
+  x <- set_repaired_names(x, repair_hint = TRUE, .name_repair, call = call)
+  x <- check_valid_cols(x, call = call)
+  x <- recycle_columns(x, .rows, lengths)
+  x
 }
 
-compat_name_repair <- function(.name_repair, validate, .missing_name_repair) {
-  if (is.null(validate)) return(.name_repair)
-
-
-  if (!.missing_name_repair) {
-    name_repair <- .name_repair
-  } else if (isTRUE(validate)) {
-    name_repair <- "check_unique"
-  } else {
-    name_repair <- "minimal"
-  }
-
-  deprecate_soft("2.0.0", "tibble::as_tibble(validate = )", "as_tibble(.name_repair =)",
-    env = foreign_caller_env()
-  )
-
-  name_repair
-}
-
-check_valid_cols <- function(x, pos = NULL) {
+check_valid_cols <- function(x, pos = NULL, call = caller_env()) {
   names_x <- names2(x)
 
   is_xd <- which(!map_lgl(x, is_valid_col))
   if (has_length(is_xd)) {
     classes <- map_chr(x[is_xd], friendly_type_of)
-    cnd_signal(error_column_scalar_type(names_x[is_xd], pos[is_xd], classes))
+    abort_column_scalar_type(names_x[is_xd], pos[is_xd], classes, call)
   }
 
   # 657
@@ -158,7 +153,7 @@ recycle_columns <- function(x, .rows, lengths) {
   if (is_empty(different_len)) return(new_tibble(x, nrow = nrow, subclass = NULL))
 
   if (any(lengths[different_len] != 1)) {
-    cnd_signal(error_incompatible_size(.rows, names(x), lengths, "Requested with `.rows` argument"))
+    abort_incompatible_size(.rows, names(x), lengths, "Requested with `.rows` argument")
   }
 
   if (nrow != 1L) {
@@ -253,7 +248,7 @@ as_tibble.table <- function(x, `_n` = "n", ..., n = `_n`, .name_repair = "check_
 #' @rdname as_tibble
 as_tibble.NULL <- function(x, ...) {
   if (missing(x)) {
-    deprecate_soft("3.0.0", "as_tibble(x = 'can\\'t be missing')")
+    deprecate_stop("3.0.0", "as_tibble(x = 'can\\'t be missing')")
   }
 
   new_tibble(list(), nrow = 0)
@@ -284,9 +279,9 @@ as_tibble.default <- function(x, ...) {
 #' as_tibble_row(list(c = "three", d = list(4:5)))
 #' as_tibble_row(1:3, .name_repair = "unique")
 as_tibble_row <- function(x,
-                          .name_repair = c("check_unique", "unique", "universal", "minimal")) {
+                          .name_repair = c("check_unique", "unique", "universal", "minimal", "unique_quiet", "universal_quiet")) {
   if (!vec_is(x)) {
-    cnd_signal(error_as_tibble_row_vector(x))
+    abort_as_tibble_row_vector(x)
   }
 
   names <- vectbl_names2(x, .name_repair = .name_repair)
@@ -305,16 +300,17 @@ as_tibble_row <- function(x,
   new_tibble(slices, nrow = 1)
 }
 
-check_all_lengths_one <- function(x) {
+check_all_lengths_one <- function(x, call = caller_env()) {
   sizes <- col_lengths(x)
 
   bad_lengths <- which(sizes != 1)
   if (!is_empty(bad_lengths)) {
-    cnd_signal(error_as_tibble_row_size_one(
+    abort_as_tibble_row_size_one(
       seq_along(x)[bad_lengths],
       names2(x)[bad_lengths],
-      sizes[bad_lengths]
-    ))
+      sizes[bad_lengths],
+      call
+    )
   }
 }
 
@@ -346,8 +342,9 @@ matrixToDataFrame <- function(x) {
 
 # Errors ------------------------------------------------------------------
 
-error_column_scalar_type <- function(names, positions, classes) {
-  tibble_error(
+abort_column_scalar_type <- function(names, positions, classes, call = caller_env()) {
+  tibble_abort(
+    call = call,
     problems(
       "All columns in a tibble must be vectors:",
       x = paste0("Column ", name_or_pos(names, positions), " is ", classes)
@@ -356,17 +353,17 @@ error_column_scalar_type <- function(names, positions, classes) {
   )
 }
 
-error_as_tibble_row_vector <- function(x) {
-  tibble_error(paste0(
+abort_as_tibble_row_vector <- function(x, call = caller_env()) {
+  tibble_abort(call = call, paste0(
     "`x` must be a vector in `as_tibble_row()`, not ", class(x)[[1]], "."
   ))
 }
 
-error_as_tibble_row_size_one <- function(j, name, size) {
+abort_as_tibble_row_size_one <- function(j, name, size, call = caller_env()) {
   desc <- tick(name)
   desc[name == ""] <- paste0("at position ", j[name == ""])
 
-  tibble_error(problems(
+  tibble_abort(call = call, problems(
     "All elements must be size one, use `list()` to wrap.",
     paste0("Element ", desc, " is of size ", size, ".")
   ))
