@@ -1,22 +1,42 @@
-#' @useDynLib tibble, .registration = TRUE
-#' @importFrom utils head tail
+## usethis namespace: start
+#' @importFrom lifecycle deprecate_soft
+#' @importFrom lifecycle deprecate_warn
+#' @importFrom lifecycle deprecate_stop
+#' @importFrom lifecycle expect_deprecated
 #' @importFrom pkgconfig set_config
-#' @import rlang
-#' @import lifecycle
-#' @import ellipsis
-#' @importFrom vctrs vec_as_location vec_as_location2 vec_as_names vec_as_names_legacy vec_c
-#' @importFrom vctrs vec_is vec_rbind vec_recycle vec_size vec_slice vec_assign
-#' @importFrom vctrs unspecified vec_as_subscript2 num_as_location vec_ptype_abbr
-#' @importFrom vctrs vec_names vec_names2 vec_set_names
-#' @importFrom vctrs new_rcrd
+#' @importFrom utils head tail
 #' @importFrom vctrs new_data_frame
-#' @aliases NULL tibble-package
+#' @importFrom vctrs new_rcrd
+#' @importFrom vctrs num_as_location
+#' @importFrom vctrs unspecified
+#' @importFrom vctrs vec_as_location
+#' @importFrom vctrs vec_as_location2
+#' @importFrom vctrs vec_as_names
+#' @importFrom vctrs vec_as_names_legacy
+#' @importFrom vctrs vec_as_subscript
+#' @importFrom vctrs vec_as_subscript2
+#' @importFrom vctrs vec_assign
+#' @importFrom vctrs vec_c
+#' @importFrom vctrs vec_is
+#' @importFrom vctrs vec_names
+#' @importFrom vctrs vec_names2
+#' @importFrom vctrs vec_ptype_abbr
+#' @importFrom vctrs vec_rbind
+#' @importFrom vctrs vec_recycle
+#' @importFrom vctrs vec_set_names
+#' @importFrom vctrs vec_size
+#' @importFrom vctrs vec_slice
+#' @import rlang
+## usethis namespace: end
+NULL
+
+#' @useDynLib tibble, .registration = TRUE
 #' @details
 #' `r lifecycle::badge("stable")`
 #'
 #' The tibble package provides utilities for handling __tibbles__, where
 #' "tibble" is a colloquial term for the S3 [`tbl_df`] class. The [`tbl_df`]
-#' class is a special case of the base [`data.frame`][base::data.frame()].
+#' class is a special case of the base [`data.frame`][base::data.frame()]
 #' class, developed in response to lessons learned over many years of data
 #' analysis with data frames.
 #'
@@ -29,8 +49,9 @@
 #'
 #' General resources:
 #'   * Website for the tibble package: <https://tibble.tidyverse.org>
-#'   * [Tibbles chapter](https://r4ds.had.co.nz/tibbles.html) in *R for Data
-#'     Science*
+#'   * [Vectors chapter](https://adv-r.hadley.nz/vectors-chap.html) in *Advanced R*
+#'     (2nd edition), specifically the
+#'     [Data frames and tibbles section](https://adv-r.hadley.nz/vectors-chap.html#tibble)
 #'
 #' Resources on specific topics:
 #'   * Create a tibble: [tibble()], [as_tibble()], [tribble()], [enframe()]
